Public Class MachineDefinition
    Inherits MachineDefinitionBase
    Implements IMachineDefinition_2_0
	
	''' <summary>
	''' The version of this machine definition as shown by choosing the Version command from the Help menu of the Configure Machine Definition dialog
	''' </summary>
	Private version As System.Version =  New System.Version("1.0.0")

    ''' <summary>
    ''' Local variable to store the format for the number of decimal places after the point.
    ''' </summary>
    Private format As String
	
	''' <summary>
	''' Local variable that is used to store the configuration.
	''' </summary>
	''' <remarks>Any changes made by the user from the 'Configure Machine Definition' command are assigned to this variable.</remarks>
	Private _configuration As Configuration

#Region "Events"

	''' <summary>
	''' This event handler is invoked when this Machine Definition has finished compiling.
	''' </summary>
	''' <param name="e">Provides data for the compiled event.</param>
    ''' <remarks>Here is where you should perform any one time initialization.</remarks>
	Private Sub Complied(ByVal e As CompiledEventArgs) Implements IMachineDefinition_2_0.Compiled
		
	End Sub
 
    ''' <summary>
    ''' This event handler is invoked first when the Nc code is generated and is where you
	''' configure the general settings.
    ''' </summary>
    ''' <param name="e">Provides data for the general settings event.</param>
    Private Sub GeneralSettings(ByVal e As GeneralSettingsEventArgs) Implements IMachineDefinition_2_0.GeneralSettings

        e.ArcBreak = ArcBreak.DoNotBreak
		e.NcCodeExtension = "m"
		e.ChipToChipAtcTime = _configuration.ChipToChipAtcTime
		e.RapidPositionAccelerationDeceleration = _configuration.RapidPositionAccelerationDeceleration
		

        'Metric parts need 3 decimal places after the point, imperial 4
        If e.Metric Then
            format = "0.0##"
            e.MinimumArcLength = 0.005
            e.MinimumArcRadius = 0.005
            e.MaximumArcRadius = 10000
        Else
            format = "0.0###"
            e.MinimumArcLength = 0.0002
            e.MinimumArcRadius = 0.0002
            e.MaximumArcRadius = 400
			e.RapidTraverseRate = _configuration.RapidTraverseRate / 25.4
    	End If

    End Sub

	''' <summary>
	''' This event handler is invoked so the start of the file can be created.
	''' </summary>
	''' <param name="e">Provides data for the program start event.</param>
	''' <remarks>Here is where you should create the begining of the file, for example the % sign as used by a Fanuc.</remarks>
	Private Sub FileStart(ByVal e As FileStartEventArgs) Implements IMachineDefinition_2_0.FileStart

	End Sub


    ''' <summary>
    ''' This event handler is invoked so the start of the program can be created.
    ''' </summary>
    ''' <param name="e">Provides data for the program start event.</param>
    ''' <remarks>Here is where you should create any preparatory code.</remarks>
    Private Sub ProgramStart(ByVal e As ProgramStartEventArgs) Implements IMachineDefinition_2_0.ProgramStart
		
   		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
	       	If _configuration.OutputCycleTime Then
				NcCode.NewString("* CYCLE TIME " & New System.TimeSpan( 0, 0, e.CycleTime).TotalMinutes.ToString("0.00") & " MINUTES")
				NcCode.NewLine()
			End If
			NcCode.NewString("Dim Abs")
			NcCode.NewLine()
			If e.Metric Then
				NcCode.NewString("Unit MM")
			Else
				NcCode.NewString("Unit Inch")
			End If
		Else
			If e.Metric Then
				NcCode.NewString("G90 G71 G17")
			Else
				NcCode.NewString("G90 G70 G17")
			End If
        	NcCode.NewLine()
	       	If _configuration.OutputCycleTime Then
				NcCode.NewString("*CYCLE TIME " & New System.TimeSpan( 0, 0, e.CycleTime).TotalMinutes.ToString("0.00") & " MINUTES")
			End If
			NcCode.NewLine()
			NcCode.NewString("M05 M09")
			NcCode.NewLine()
			NcCode.NewString("G28Z")
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the tool change event.</param>
    Private Sub ToolChange(ByVal e As ToolChangeEventArgs) Implements IMachineDefinition_2_0.ToolChange
		
		
		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Rapid Z 0.0000 Tool# 0")
			NcCode.NewLine()
	        NcCode.NewString("Tool# " & e.ToolNumber)
		Else
	        NcCode.NewString("T" & e.ToolNumber & " *" & e.ToolName)
		End If
    	   NcCode.NewLine()
		
    End Sub

    ''' <summary>
    ''' This event handler is invoked before every tool change, unless it is the first, where it is not called.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel tool change event.</param>
    Private Sub CancelToolChange(ByVal e As CancelToolChangeEventArgs) Implements IMachineDefinition_2_0.CancelToolChange

		If _configuration.ProgramFormat = ProgramFormat.GCode Then
			NcCode.NewString("G28Z")
			NcCode.NewLine()
		End If
	
	End Sub

    ''' <summary>
    ''' This event handler is invoked for the first position after a tool change for Profiling and Pocketing operations.
    ''' </summary>
    ''' <param name="e">Provides data for the first position after tool change event.</param>
    Private Sub FirstPositionAfterToolChange(ByVal e As FirstPositionAfterToolChangeEventArgs) Implements IMachineDefinition_2_0.FirstPositionAfterToolChange

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Rapid      ")
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.PositionZ.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G0", Modal.Off)
			NcCode.NewString("G90 ")
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewString(e.ToolDirection & " S" & e.SpindleSpeed)
			NcCode.NewLine()
			NcCode.NewDimensionedWord("Z", e.PositionZ.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewString(e.Coolant)
		End If
		NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked for all Profiling and Pocketing rapid positions, other than
    ''' the <see cref="FirstPositionAfterToolChange"/>.
    ''' </summary>
    ''' <param name="e">Provides data for the rapid position event.</param>
    Private Sub RapidPositiong(ByVal e As RapidPositionEventArgs) Implements IMachineDefinition_2_0.RapidPositioning

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Rapid      ")
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G0", Modal.On)
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every linear interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the linear interpolation event.</param>
    Private Sub LinearInterpolation(ByVal e As LinearInterpEventArgs) Implements IMachineDefinition_2_0.LinearInterpolation

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Line       ")
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G1", Modal.On)
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()
	
    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the left.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC left event.</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves.</remarks>
    Private Sub ApplyCRCLeft(ByVal e As ApplyCRCLeftEventArgs) Implements IMachineDefinition_2_0.ApplyCRCLeft

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Line       ")
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewString("ToolComp Left ")
			NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G41", Modal.Off)
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is applied on the right.
    ''' </summary>
    ''' <param name="e">Provides data for the apply CRC right event</param>
    ''' <remarks>This is a linear move. CRC is not applied on circular interpolation moves</remarks>
    Private Sub ApplyCRCRight(ByVal e As ApplyCRCRightEventArgs) Implements IMachineDefinition_2_0.ApplyCRCRight

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Line       ")
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewString("ToolComp Right ")
			NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G42", Modal.Off)
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()

	End Sub

    ''' <summary>
    ''' This event handler is invoked when Cutter Radius Compensation (CRC) is canceled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel CRC right event.</param>
    ''' <remarks>This is a linear move. CRC is not cancelled on circular interpolation moves.</remarks>
    Private Sub CancelCRC(ByVal e As CancelCRCEventArgs) Implements IMachineDefinition_2_0.CancelCRC

        If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Line       ")
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewString("ToolComp Off ")
			NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G40", Modal.Off)
			NcCode.NewDimensionedWord("X", e.AbsX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()
	
    End Sub

    ''' <summary>
    ''' This event handler is invoked for every counter clockwise circular interpolation move.
    ''' </summary>
    ''' <param name="e">Provides data for the clockwise circular interpolation event</param>
    Private Sub CWInterpolation(ByVal e As CWInterpolationEventArgs) Implements IMachineDefinition_2_0.CWInterpolation

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Arc Cw     ")
			NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewString("XCenter " & e.AbsCentreX.ToString(format, CultureInfo))
			NcCode.NewString(" YCenter " & e.AbsCentreY.ToString(format, CultureInfo) & " ")
			NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G2", Modal.On)
			NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("I", e.IncrementalCentreX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("J", e.IncrementalCentreY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked for every clockwise circular interpolation move
    ''' </summary>
    ''' <param name="e">Provides data for the circular interpolation event.</param>
    Private Sub CCWInterpolation(ByVal e As CCWInterpolationEventArgs) Implements IMachineDefinition_2_0.CCWInterpolation

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Arc Ccw    ")
			NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewString("XCenter " & e.AbsCentreX.ToString(format, CultureInfo))
			NcCode.NewString(" YCenter " & e.AbsCentreY.ToString(format, CultureInfo) & " ")
			NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		Else
			NcCode.NewWord("G3", Modal.On)
			NcCode.NewDimensionedWord("X", e.AbsEndX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.AbsEndY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.AbsEndZ.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("I", e.IncrementalCentreX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("J", e.IncrementalCentreY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("F", e.FeedRate.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole after a tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole after toolchange event</param>
    Private Sub DrillFirstHoleAfterToolChange(ByVal e As DrillFirstHoleAfterToolChangeEventArgs) Implements IMachineDefinition_2_0.DrillFirstHoleAfterToolChange

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			If e.DrillingType = DrillingType.ChipBreaker
				MessageBox.Show("Chip Breaker not supported", "Drilling", MessageBoxButtons.OK, MessageBoxIcon.Information)
				Return
			Else If e.DrillingType = DrillingType.Reaming
				MessageBox.Show("Reaming not supported", "Drilling", MessageBoxButtons.OK, MessageBoxIcon.Information)
				Return
			End If
		
			If e.DrillingType = DrillingType.PeckDrill
				NcCode.NewString("PeckDrill ")
			Else If e.DrillingType = DrillingType.Tapping Or e.DrillingType = DrillingType.RigidTapping
				NcCode.NewString("Tapping ")
			Else If e.DrillingType = DrillingType.Boring
				NcCode.NewString("Boring ")
			Else
				NcCode.NewString("BasicDrill ")		
			End If
			NcCode.NewString("ZDepth " & e.FinishDepth.ToString(format, CultureInfo) & " ")
			NcCode.NewString("StartHgt " & e.IntialRapid)
			If e.RetractToIntialRapid Then
				NcCode.NewString(" ReturnHgt " & e.IntialRapid)		
			Else
				NcCode.NewString(" ReturnHgt " & e.FeedFrom)		
			End If
			If e.DrillingType = DrillingType.PeckDrill
				NcCode.NewString(" Peck " & e.PeckAmount)		
			Else If e.DrillingType = DrillingType.Boring And e.Dwell > 0
				NcCode.NewString(" Dwell " & e.Dwell)
			Else If e.DrillingType = DrillingType.Tapping
				NcCode.NewString(" TPIorLead " & e.Pitch)
			End If
			If e.DrillingType = DrillingType.RigidTapping
				NcCode.NewString(" SyncSpin Yes")
			End If
			If e.DrillingType <> DrillingType.Tapping And e.DrillingType <> DrillingType.RigidTapping
				NcCode.NewString(" ")
				NcCode.NewDimensionedWord("Feed ", e.FeedRate.ToString(format, CultureInfo), Modal.Off)		
			End If
			NcCode.NewLine()
			NcCode.NewString("           ")
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)		
		Else
			If e.DrillingType = DrillingType.ChipBreaker
				MessageBox.Show("Chip Breaker not supported", "Drilling", MessageBoxButtons.OK, MessageBoxIcon.Information)
				Return
			End If
			NcCode.NewWord("G0", Modal.Off)
			NcCode.NewString("G90 ")
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewString(e.ToolDirection & " S" & e.SpindleSpeed)
			NcCode.NewLine()
	
			NcCode.NewDimensionedWord("Z", e.IntialRapid.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewString(e.Coolant)
			NcCode.NewLine()
	
			Dim feedrate As Double
			If e.DrillingType = DrillingType.Drilling And e.Dwell = 0 Then
				NcCode.NewString("G81 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.Drilling And e.Dwell > 0 Then
				NcCode.NewString("G82 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.PeckDrill Then
				NcCode.NewString("G83 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.Tapping Or e.DrillingType = DrillingType.RigidTapping Then
				NcCode.NewString("G84 ")
				feedrate = e.Pitch * e.SpindleSpeed * 0.95 'Reduce feedrate by 5% for floating holder
			ElseIf e.DrillingType = DrillingType.Boring Or e.DrillingType = DrillingType.Reaming Then
				NcCode.NewString("G85 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.CustomCycle1 Then
				'Add code for additional drilling cycle
			End If
			NcCode.NewDimensionedWord("Z", e.FinishDepth.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewString("R" & e.FeedFrom.ToString(format, CultureInfo) & " ")
			If e.DrillingType = DrillingType.Tapping Or e.DrillingType = DrillingType.RigidTapping Then
				If e.Metric
					NcCode.NewString("F" & e.Pitch & " ")					
				Else
					NcCode.NewString("F" & (CInt(25.4/e.Pitch)) & " ")					
				End If
				If e.DrillingType = DrillingType.Tapping Then 
					NcCode.NewString("S0 ")					
				Else
					NcCode.NewString("S1 ")					
				End If
			Else
				NcCode.NewDimensionedWord("F", feedrate.ToString(format, CultureInfo), Modal.Off)
			End If
			If e.Dwell > 0 Then
				NcCode.NewString("D" & e.Dwell & " ")
			End If
			If e.DrillingType = DrillingType.PeckDrill Then
				NcCode.NewString("I" & e.PeckAmount.ToString(format, CultureInfo) & " ")
			ElseIf e.DrillingType = DrillingType.ChipBreaker Then
				NcCode.NewString("Q" & e.PeckAmount.ToString(format, CultureInfo) & " ")
			End If
			If e.RetractToIntialRapid Then
				NcCode.NewString("P" & e.IntialRapid.ToString(format, CultureInfo) & " ")
			End If
			NcCode.NewLine()
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill the first hole in an operation that is NOT after a Tool change.
    ''' </summary>
    ''' <param name="e">Provides data for the drill first hole in operation event.</param>
    Private Sub DrillFirstHoleInOperation(ByVal e As DrillFirstHoleInOperationEventArgs) Implements IMachineDefinition_2_0.DrillFirstHoleInOperation

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			DrillFirstHoleAfterToolChange(New DrillFirstHoleAfterToolChangeEventArgs(e.PositionX, e.PositionY, e.DrillingType, _
																					String.Empty, New ToolType(), 0, 0, _
																					String.Empty, 0, String.Empty, 0 , _
																					e.Coolant, 0, String.Empty, _
																					e.FeedRate, e.SpindleSpeed, _
																					e.IntialRapid, e.FeedFrom, _
																					e.MaterialSurface, e.FinishDepth, _
																					e.PeckAmount, e.Dwell, e.Pitch, _
																					e.RetractToIntialRapid, e.CoolantType, _
																					e.SpindleDirection, e.Metric))
		Else
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Z", e.IntialRapid.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewLine()
	
			Dim feedrate As Double
			If e.DrillingType = DrillingType.Drilling And e.Dwell = 0 Then
				NcCode.NewString("G81 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.Drilling And e.Dwell > 0 Then
				NcCode.NewString("G82 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.PeckDrill Then
				NcCode.NewString("G83 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.Tapping Or e.DrillingType = DrillingType.RigidTapping Then
				NcCode.NewString("G84 ")
				feedrate = e.Pitch * e.SpindleSpeed * 0.95 'Reduce feedrate by 5% for floating holder
			ElseIf e.DrillingType = DrillingType.Boring Or e.DrillingType = DrillingType.Reaming Then
				NcCode.NewString("G85 ")
				feedrate = e.FeedRate
			ElseIf e.DrillingType = DrillingType.CustomCycle1 Then
				'Add code for additional drilling cycle
			End If
			NcCode.NewDimensionedWord("Z", e.FinishDepth.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewString("R" & e.FeedFrom.ToString(format, CultureInfo) & " ")
			If e.DrillingType = DrillingType.Tapping Or e.DrillingType = DrillingType.RigidTapping Then
				If e.Metric
					NcCode.NewString("F" & e.Pitch & " ")					
				Else
					NcCode.NewString("F" & (CInt(25.4/e.Pitch)) & " ")					
				End If
				If e.DrillingType = DrillingType.Tapping Then 
					NcCode.NewString("S0 ")					
				Else
					NcCode.NewString("S1 ")					
				End If
			Else
				NcCode.NewDimensionedWord("F", feedrate.ToString(format, CultureInfo), Modal.Off)
			End If
			If e.Dwell > 0 Then
				NcCode.NewString("D" & e.Dwell & " ")
			End If
			If e.DrillingType = DrillingType.PeckDrill Then
				NcCode.NewString("I" & e.PeckAmount.ToString(format, CultureInfo) & " ")
			ElseIf e.DrillingType = DrillingType.ChipBreaker Then
				NcCode.NewString("Q" & e.PeckAmount.ToString(format, CultureInfo) & " ")
			End If
			If e.RetractToIntialRapid Then
				NcCode.NewString("P" & e.IntialRapid.ToString(format, CultureInfo) & " ")
			End If
			NcCode.NewLine()
		End If
		NcCode.NewLine()																			

    End Sub

    ''' <summary>
    ''' This event handler is invoked to drill a hole that is NOT after a Tool change or the first in the operation.
    ''' </summary>
    ''' <param name="e">Provides data for the drill hole event.</param>
    Private Sub DrillHole(ByVal e As DrillHoleEventArgs) Implements IMachineDefinition_2_0.DrillHole

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("           ")
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.Off)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.Off)
		Else
			NcCode.NewDimensionedWord("X", e.PositionX.ToString(format, CultureInfo), Modal.On)
			NcCode.NewDimensionedWord("Y", e.PositionY.ToString(format, CultureInfo), Modal.On)
		End If
		NcCode.NewLine()

    End Sub

    ''' <summary>
    ''' This event handler is invoked when a drilling cycle needs to be canelled.
    ''' </summary>
    ''' <param name="e">Provides data for the cancel drilling cycle event.</param>
    Private Sub CancelDrillingCycles(ByVal e As CancelDrillingCyclesEventArgs) Implements IMachineDefinition_2_0.CancelDrillingCycles

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("DrillOff")
		Else
			NcCode.NewString("G80")
		End If
		NcCode.NewLine()
		
    End Sub

    ''' <summary>
    ''' This event handler is invoked last.
    ''' </summary>
    Private Sub ProgramEnd(ByVal e As ProgramEndEventArgs) Implements IMachineDefinition_2_0.ProgramEnd

		If _configuration.ProgramFormat = ProgramFormat.Conversational Then
			NcCode.NewString("Rapid Z 0.0000 Tool# 0")
			NcCode.NewLine()
			NcCode.NewString("EndMain")
			NcCode.NewLine()
		Else
			NcCode.NewString("G28Z")
			NcCode.NewLine()
			NcCode.NewString("M2")
			NcCode.NewLine()
		End If

    End Sub

	''' <summary>
	''' <summary>
	''' This event handler is invoked so the start of the file can be created.
	''' </summary>
	''' <param name="e">Provides data for the program start event.</param>
	''' <remarks>Here is where you should create the end of the file, for example the % sign as used by a Fanuc.</remarks>
	Private Sub FileEnd(ByVal e As FileEndEventArgs) Implements IMachineDefinition_2_0.FileEnd
   		
		If _configuration.ProgramFormat = ProgramFormat.GCode And _configuration.AppendAtSymbol Then
			For Each line As System.Text.StringBuilder In e.NcCodeLines
				line.Insert(0, "@")
            Next
		End If

	End Sub
	''' <summary>
	''' This event handler is invoked to define the start of all Sub Routines.
	''' </summary>
	''' <param name="e">Provides data for the all sub routine start event.</param>
	Private Sub SubRoutineAllStart(ByVal e As SubRoutineAllStartEventArgs) Implements IMachineDefinition_2_0.SubRoutineAllStart

	End Sub

	''' <summary>
	''' This event handler is invoked when a Sub Routine is called.
	''' </summary>
	''' <param name="e">Provides data for the all sub routine start event.</param>
	Private Sub SubRoutineCall(ByVal e As SubRoutineCallEventArgs) Implements IMachineDefinition_2_0.SubRoutineCall

			System.Windows.Forms.MessageBox.Show("Sub Routines not supported", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
		
	End Sub
	
    ''' <summary>
	''' This event handler is invoked to define the start of the Sub Routine.
	''' </summary>
	''' <param name="e">Provides data for the sub routine start event.</param>
	Private Sub SubRoutineStart(ByVal e As SubRoutineStartEventArgs) Implements IMachineDefinition_2_0.SubRoutineStart

	End Sub
	''' <summary>
	''' This event handler is invoked to define the end of the Sub Routine.
	''' </summary>
	''' <param name="e">Provides data for the sub routine end event.</param>
	Private Sub SubRoutineEnd(ByVal e As SubRoutineEndEventArgs) Implements IMachineDefinition_2_0.SubRoutineEnd

	End Sub
	''' <summary>
	''' This event handler is invoked to define the end of all Sub Routines.
	''' </summary>
	''' <param name="e">Provides data for the all sub routine end event.</param>
	Private Sub SubRoutineAllEnd(ByVal e As SubRoutineAllEndEventArgs) Implements IMachineDefinition_2_0.SubRoutineAllEnd
	End Sub


#End Region

#Region "User Congigured Input"

    ''' <summary>
    ''' When the user clicks the 'User Configured Input' button on the NC Code tab in the Part Manager,
    ''' this sub procedure is called. Here we display the dialog box where the user can input information
	''' that can be used in the Machine Definition.
    ''' </summary>
    Private Sub UserConfiguredInput(ByVal e As UserConfiguredInputEventArgs) Implements IMachineDefinition_2_0.UserConfiguredInput

    End Sub

#End Region

#Region "Properties"

    ''' <summary>
    ''' Gets the Nc code to be output for a clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CWSpindleStart() As String Implements IMachineDefinition_2_0.CWSpindleStart
        Get
            Return "M3"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for a counter clockwise spindle start.
    ''' </summary>
    Private ReadOnly Property CCWSpindleStart() As String Implements IMachineDefinition_2_0.CCWSpindleStart
        Get
            Return "M4"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for flood coolant.
    ''' </summary>
    Private ReadOnly Property CoolantFlood() As String Implements IMachineDefinition_2_0.CoolantFlood
        Get
            Return "M8"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through spindle.
    ''' </summary>
    Private ReadOnly Property CoolantThroughSpindle() As String Implements IMachineDefinition_2_0.CoolantThroughSpindle
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant through the tool.
    ''' </summary>
    Private ReadOnly Property CoolantThroughTool() As String Implements IMachineDefinition_2_0.CoolantThroughTool
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for mist coolant.
    ''' </summary>
    Private ReadOnly Property CoolantMist() As String Implements IMachineDefinition_2_0.CoolantMist
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for an air blast.
    ''' </summary>
    Private ReadOnly Property CoolantAirBlast() As String Implements IMachineDefinition_2_0.CoolantAirBlast
        Get
            Return ""
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for coolant off.
    ''' </summary>
    Private ReadOnly Property CoolantNone() As String Implements IMachineDefinition_2_0.CoolantNone
        Get
            Return "M9"
        End Get
    End Property

    ''' <summary>
    ''' Gets the Nc code to be output for custom coolant.
    ''' </summary>
    Private ReadOnly Property CoolantCustom() As String Implements IMachineDefinition_2_0.CoolantCustom
        Get
            Return ""
        End Get
    End Property

#End Region

#Region "Modals"

    ''' <summary>
    ''' This event handler is invoked to sets up modal words
    ''' </summary>
    ''' <param name="e">Provides data for the modals event</param>
    Private Sub Modals(ByVal e As ModalEventArgs) Implements IMachineDefinition_2_0.Modals

        'Firstly create a new 'ModalWordGroup'.
        Dim wordGroup1 As New ModalWordGroup()
        'Add Words to 'wordGroup1'.
        wordGroup1.Words.Add("G0")
        wordGroup1.Words.Add("G1")
        wordGroup1.Words.Add("G2")
        wordGroup1.Words.Add("G3")

        'Finally add the groups to 'WordGroups'.
        e.WordGroups.Add(wordGroup1)

        'Add Absolute Modal Dimentioned Words to 'AbsoluteDimensionedWords'.	
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("X"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Y"))
        e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Z"))
		If _configuration.ProgramFormat = ProgramFormat.GCode Then
        	e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("F"))
		Else
        	e.AbsoluteDimensionedWords.Add(New ModalAbsoluteDimensionedWord("Feed "))
		End If

        'Add Incremental Modal Dimentioned Words to 'IncrementalDimensionedWords'.		
        e.IncrementalDimensionedWords.Add(New ModalIncrementalDimensionedWord("I"))
        e.IncrementalDimensionedWords.Add(New ModalIncrementalDimensionedWord("J"))

    End Sub

#End Region

End Class

#Region "Configuration"

''' <summary>
''' Class to manage the configuration of this Machine Definition.
''' </summary>
Public Class Configuration
	
	Private _programFormat As ProgramFormat
	Private _appendAtSymbol As Boolean
	Private _chipToChipAtcTime As Double
	Private _rapidTraverseRate As Double
	Private _rapidPositionAccelerationDeceleration As Double
	Private _outputCycleTime As Boolean
	
	''' <summary>
	''' Gets or sets the format of the program.
	''' </summary>
	<System.ComponentModel.Description("The format of the program."), _
	System.ComponentModel.DisplayName("Program Format")> _
	Public Property ProgramFormat() As ProgramFormat
        Get
            Return _programFormat
        End Get
        Set
            _programFormat = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets a value indicating if an @ symbol should be appended to the begining of a G-Code line.
	''' As used by the 3300 control.
	''' </summary>
	<System.ComponentModel.Description("Should an @ symbol should be appended to the begining of a G-Code line? As used by the 3300 control."), _
	System.ComponentModel.DisplayName("Append @ symbol")> _
	Public Property AppendAtSymbol() As Boolean
        Get
            Return _appendAtSymbol
        End Get
        Set
            _appendAtSymbol = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets a value indicating if the cycle time should be output in a comment.
	''' </summary>
	<System.ComponentModel.Description("Should the cycle time be output?"), _
	System.ComponentModel.DisplayName("Output Cycle Time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property OutputCycleTime() As Boolean
        Get
            Return _outputCycleTime
        End Get
        Set
            _outputCycleTime = value
        End Set
    End Property

	''' <summary>
	''' Gets or sets the chip to chip tool change time in seconds.
	''' </summary>
	<System.ComponentModel.Description("The chip to chip tool change time in seconds"), _
	System.ComponentModel.DisplayName("Chip to chip tool change time"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property ChipToChipAtcTime() As Double
        Get
            Return _chipToChipAtcTime
        End Get
        Set
            _chipToChipAtcTime = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the rapid traverse rate in millimetres/minute.
	''' </summary>
	<System.ComponentModel.Description("The rapid traverse rate in millimetres/minute. This value is converted to imperial for Parts with Inch units."), _
	System.ComponentModel.DisplayName("Rapid traverse rate"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidTraverseRate() As Double
        Get
            Return _rapidTraverseRate
        End Get
        Set
            _rapidTraverseRate = value
        End Set
    End Property
	
	''' <summary>
	''' Gets or sets the time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration.
	''' </summary>
	<System.ComponentModel.Description("The time for a each Rapid move that should be added to the cycle time due to Acceleration/Deceleration"), _
	System.ComponentModel.DisplayName("Rapid Acceleration/Deceleration"), _
	System.ComponentModel.Category("Cycle Time")> _
	Public Property RapidPositionAccelerationDeceleration() As Double
        Get
            Return _rapidPositionAccelerationDeceleration

        End Get
        Set
            _rapidPositionAccelerationDeceleration = value
        End Set
    End Property

End Class

''' <summary>
''' The format of the program.
''' </summary>
Public Enum ProgramFormat

	''' <summary>
	''' G Code format.
	''' </summary>
	GCode
	
	''' <summary>
	''' Conversational format.
	''' </summary>
	Conversational
	
End Enum

#End Region